#include <initializer_list>
#include <vector>

template<typename T>
class ndarray {
private:
    std::vector<int> dims;
    std::vector<T> data;
    T* point(int next_axis, T* start) {return start;}
    template<typename S, typename... Tail>
    T* point(int next_axis, T* start, S next_index, Tail... other_indices) {
        int shift = next_index;
        for (int k=next_axis+1; k < dims.size(); ++k) {
            shift *= dims[k];
        }
        return point(next_axis + 1, start + shift, other_indices...);
    }
public:
    using value_type = T;
    ndarray() {} // need separate default; "empty initializer list" doesn't count
    ndarray(std::initializer_list<int> d)
    : dims{d} {
        int total_elem = 1;
        for (int n : dims) {total_elem *= n;}
        data.resize(total_elem);
    }
    template<typename... Indices>
    T& get(Indices... inds) {
        return *point(0, &data[0], inds...);
    }
};
